@@ -21,26 +21,26 @@ mkde <- function(x, h, thumb = "none") {
   d <- ncol(x)  ## sample and dimensionality of x
 
   if (thumb == "silverman") {
-    s <- apply(x, 2, sd)
-    h <- (4/(d + 2))^(1/(d + 4)) * s * n^(-1/(d + 4))
+    s <- fastR::colVars(x, std = TRUE)
+    h <- ( 4/(d + 2) )^( 1/(d + 4) ) * s * n^( -1/(d + 4) )
 
   } else  if (thumb == "scott") {
-    s <- apply(x, 2, sd)
-    h <- s * n^(-1/(d + 4))
+    s <- fastR::colVars(x, std = TRUE)
+    h <- s * n^( -1/(d + 4) )
+
   } else if (thumb == "estim") {
     h <- mkde.tune(x)$hopt
-  }
 
-  else  h <- h
+  } else  h <- h
 
   if (length(h) == 1) {
-    h <- diag(h, d)
+    h <- diag( 1/h, d )
   } else h <- diag(h)
 
-  con <- prod( diag(h) )
-  y <- x %*% solve(h)
-  a1 <- dist(y, diag = TRUE, upper = TRUE)
-  a1 <- as.matrix(a1)
+  con <- prod( diag( 1 / h ) )
+  y <- x %*% h
+  a1 <- fields::rdist(y, compact = FALSE)
   f <- as.vector( 1/(2 * pi)^(d/2) * (1/con) * rowMeans( exp(-0.5 * a1^2 ) ) )
   f
+
 }
