@@ -13,6 +13,7 @@ pcr <- function(y, x, k = 1, xnew = NULL) {
   ## y is the univariate dependent variable
   ## x contains the independent variables
   ## k shows the number of components to keep
+
   x <- as.matrix(x)
   y <- as.vector(y)
   m <- mean(y)
@@ -20,9 +21,10 @@ pcr <- function(y, x, k = 1, xnew = NULL) {
   n <- nrow(x)
   p <- ncol(x)
   mx <- colMeans(x)
-  s <- apply(x, 2, sd)
-  s <- diag(1/s)
-  x <- scale(x)[1:n, ]  ## standardize the independent variables
+  s <- fastR::colVars(x, std = TRUE)
+  x <- ( t(x) - m )/ s  ## standardise the x values
+  x <- t(x)
+
   eig <- eigen( crossprod(x) )  ## eigen analysis of the design matrix
   values <- eig$values  ## eigenvalues
   per <- cumsum( values / sum(values) )  ## cumulative proportion of each eigenvalue
@@ -39,19 +41,23 @@ pcr <- function(y, x, k = 1, xnew = NULL) {
     xnew <- as.matrix(xnew)
     xnew <- matrix(xnew, ncol = p)
     nu <- nrow(xnew)
-    xnew <- ( xnew - rep(mx, rep(nu, p)) ) %*% s  ## standardize the xnew values
+    xnew <- ( t(xnew) - mx) / s  ## standardize the xnew values
+    xnew <- t(xnew)
     est <- as.vector( m + xnew %*% b )  ## predicted values for PCA model
   } else {
     est <- as.vector( m + x %*% b )  ## fitted values for PCA model
     mse <- sum( (y + m - est)^2 ) / (n - k)  ## mean squared error of PCA model
     r2 <- 1 - (n - 1)/(n - k - 1) * ( 1 - cor(y + m, est)^2 )
-  }  
+  }
+
   ## rs is the adjusted R squared for the PCA model
   va <- sigma * vec[, 1:k] %*% solve(zzk, t(vec[, 1:k]) )
   ## va is the covariance matrix of the parameters
   ## of the parameters of the PCA model
   vara <- sqrt( diag(va) )  ## standard errors of coefficients of PCA model
+
   param <- cbind(b, vara)
   colnames(param) <- c("beta", "std.error")
   list(parameters = param, mse = mse, adj.rsq = r2, per = per[k], est = est)
-}
\ No newline at end of file
+
+}
